@@ -1,4 +1,5 @@
 import os
+from string import Template
 
 kubeconfig_temp_path = os.path.join('.kubedev', 'kube_config_tmp')
 
@@ -82,9 +83,12 @@ class KubedevConfig:
     tag = KubedevConfig.get_tag(env_accessor)
     imageRegistry = kubedev["imageRegistry"]
     name = kubedev["name"]
+    globalUsedFrameworks = kubedev["usedFrameworks"] if "usedFrameworks" in kubedev else []
     if "deployments" in kubedev:
       for deploymentKey, deployment in kubedev["deployments"].items():
         finalDeploymentName = KubedevConfig.collapse_names(name, deploymentKey)
+        localUsedFrameworks = deployment["usedFrameworks"] if "usedFrameworks" in deployment else []
+        usedFrameworks = globalUsedFrameworks + localUsedFrameworks
         images[deploymentKey] = {
             "imageName": f"{imageRegistry}/{finalDeploymentName}:{tag}",
             "imageNameTagless": f"{imageRegistry}/{finalDeploymentName}",
@@ -92,7 +96,8 @@ class KubedevConfig:
             "ports": deployment['ports'] if 'ports' in deployment else dict(),
             "buildEnvs": {*globalBuildEnvs, *KubedevConfig.load_envs(deployment, True, False)},
             "containerEnvs": {*globalContainerEnvs, *KubedevConfig.load_envs(deployment, False, True)},
-            "volumes": deployment["volumes"]["dev"] if "volumes" in deployment and "dev" in deployment["volumes"] else dict()
+            "volumes": deployment["volumes"]["dev"] if "volumes" in deployment and "dev" in deployment["volumes"] else dict(),
+            "usedFrameworks": usedFrameworks
         }
     return images
 
@@ -176,3 +181,7 @@ class KubedevConfig:
       return kubedev['helmReleaseName']
     else:
       return kubedev['name']
+
+  @staticmethod
+  def expand_variables(text: str, env_accessor) -> str:
+    return Template(text).substitute(env_accessor.environ())
